@@ -20,12 +20,15 @@ import decimal
 from ceilometerclient import client as cclient
 from keystoneauth1 import loading as ks_loading
 from oslo_config import cfg
+from oslo_log import log as logging
 from oslo_utils import units
 
 from cloudkitty import collector
 from cloudkitty import utils as ck_utils
 
 
+LOG = logging.getLogger(__name__)
+
 CEILOMETER_COLLECTOR_OPTS = 'ceilometer_collector'
 ceilometer_collector_opts = ks_loading.get_auth_common_conf_options()
 
@@ -38,6 +41,8 @@ ks_loading.register_auth_conf_options(
     CEILOMETER_COLLECTOR_OPTS)
 CONF = cfg.CONF
 
+METRICS_CONF = ck_utils.get_metrics_conf(CONF.collect.metrics_conf)
+
 
 class ResourceNotFound(Exception):
     """Raised when the resource doesn't exist."""
@@ -80,11 +85,12 @@ class CeilometerCollector(collector.BaseCollector):
 
     units_mappings = {
         'compute': 'instance',
-        'image': 'MB',
-        'volume': 'GB',
+        'image': 'MiB',
+        'volume': 'GiB',
         'network.bw.out': 'MB',
         'network.bw.in': 'MB',
         'network.floating': 'ip',
+        'radosgw.usage': 'GiB'
     }
 
     def __init__(self, transformers, **kwargs):
@@ -113,7 +119,16 @@ class CeilometerCollector(collector.BaseCollector):
         try:
             info["metadata"].extend(transformers['CeilometerTransformer']
                                     .get_metadata(resource_name))
-            info["unit"] = cls.units_mappings[resource_name]
+
+            try:
+                tmp = METRICS_CONF['metrics_units'][resource_name]
+                info['unit'] = list(tmp.values())[0]['unit']
+            # NOTE(mc): deprecated except part kept for backward compatibility.
+            except KeyError:
+                LOG.warning('Error when trying to use yaml metrology conf.')
+                LOG.warning('Fallback on the deprecated oslo config method.')
+                info['unit'] = cls.units_mappings[resource_name]
+
         except KeyError:
             pass
         return info
@@ -201,9 +216,23 @@ class CeilometerCollector(collector.BaseCollector):
                                                  instance)
             instance = self._cacher.get_resource_detail('compute',
                                                         instance_id)
-            compute_data.append(
-                self.t_cloudkitty.format_item(instance, self.units_mappings[
-                    "compute"], 1))
+
+            try:
+                met = list(METRICS_CONF['metrics_units']['compute'].values())
+                compute_data.append(self.t_cloudkitty.format_item(
+                    instance,
+                    met[0]['unit'],
+                    1,
+                ))
+            # NOTE(mc): deprecated except part kept for backward compatibility.
+            except KeyError:
+                LOG.warning('Error when trying to use yaml metrology conf.')
+                LOG.warning('Fallback on the deprecated oslo config method.')
+                compute_data.append(self.t_cloudkitty.format_item(
+                    instance,
+                    self.units_mappings['compute'],
+                    1,
+                ))
         if not compute_data:
             raise collector.NoDataCollected(self.collector_name, 'compute')
         return self.t_cloudkitty.format_service('compute', compute_data)
@@ -227,10 +256,35 @@ class CeilometerCollector(collector.BaseCollector):
             image = self._cacher.get_resource_detail('image',
                                                      image_id)
 
-            image_size_mb = decimal.Decimal(image_stats.max) / units.Mi
-            image_data.append(
-                self.t_cloudkitty.format_item(image, self.units_mappings[
-                    "image"], image_size_mb))
+            # Unit conversion
+            try:
+                conv_data = METRICS_CONF['metrics_units']['image']
+                image_size_mb = ck_utils.convert_unit(
+                    decimal.Decimal(image_stats.max),
+                    conv_data['image.size'].get('factor', 1),
+                    conv_data['image.size'].get('offset', 0),
+                )
+            except KeyError:
+                LOG.warning('Error when trying to use yaml metrology conf.')
+                LOG.warning('Fallback on the deprecated hardcoded method.')
+                image_size_mb = decimal.Decimal(image_stats.max) / units.Mi
+
+            try:
+                met = list(METRICS_CONF['metrics_units']['image'].values())
+                image_data.append(self.t_cloudkitty.format_item(
+                    image,
+                    met[0]['unit'],
+                    image_size_mb,
+                ))
+            # NOTE(mc): deprecated except part kept for backward compatibility.
+            except KeyError:
+                LOG.warning('Error when trying to use yaml metrology conf.')
+                LOG.warning('Fallback on the deprecated oslo config method.')
+                image_data.append(self.t_cloudkitty.format_item(
+                    image,
+                    self.units_mappings['image'],
+                    image_size_mb,
+                ))
 
         if not image_data:
             raise collector.NoDataCollected(self.collector_name, 'image')
@@ -255,9 +309,31 @@ class CeilometerCollector(collector.BaseCollector):
                                                  volume)
             volume = self._cacher.get_resource_detail('volume',
                                                       volume_id)
-            volume_data.append(
-                self.t_cloudkitty.format_item(volume, self.units_mappings[
-                    "volume"], volume_stats.max))
+
+            # Unit conversion
+            try:
+                conv_data = METRICS_CONF['metrics_units']['volume']
+                volume_stats.max = ck_utils.convert_unit(
+                    decimal.Decimal(volume_stats.max),
+                    conv_data['volume.size'].get('factor', 1),
+                    conv_data['volume.size'].get('offset', 0),
+                )
+
+                volume_data.append(self.t_cloudkitty.format_item(
+                    volume,
+                    conv_data['volume.size']['unit'],
+                    volume_stats.max,
+                ))
+            # NOTE(mc): deprecated except part kept for backward compatibility.
+            except KeyError:
+                LOG.warning('Error when trying to use yaml metrology conf.')
+                LOG.warning('Fallback on the deprecated oslo config method.')
+                volume_data.append(self.t_cloudkitty.format_item(
+                    volume,
+                    self.units_mappings['volume'],
+                    volume_stats.max,
+                ))
+
         if not volume_data:
             raise collector.NoDataCollected(self.collector_name, 'volume')
         return self.t_cloudkitty.format_service('volume', volume_data)
@@ -293,10 +369,35 @@ class CeilometerCollector(collector.BaseCollector):
             tap = self._cacher.get_resource_detail('network.tap',
                                                    tap_id)
 
-            tap_bw_mb = decimal.Decimal(tap_stat.max) / units.M
-            bw_data.append(
-                self.t_cloudkitty.format_item(tap, self.units_mappings[
-                    "network.bw." + direction], tap_bw_mb))
+            # Unit conversion
+            try:
+                conv = METRICS_CONF['metrics_units']['network.bw.' + direction]
+                tap_bw_mb = ck_utils.convert_unit(
+                    decimal.Decimal(tap_stat.max),
+                    conv[resource_type].get('factor', 1),
+                    conv[resource_type].get('offset', 0),
+                )
+            except KeyError:
+                LOG.warning('Error when trying to use yaml metrology conf.')
+                LOG.warning('Fallback on the deprecated hardcoded method.')
+                tap_bw_mb = decimal.Decimal(tap_stat.max) / units.M
+
+            try:
+                met = METRICS_CONF['metrics_units']['network.bw.' + direction]
+                bw_data.append(self.t_cloudkitty.format_item(
+                    tap,
+                    list(met.values())[0]['unit'],
+                    tap_bw_mb,
+                ))
+            # NOTE(mc): deprecated except part kept for backward compatibility.
+            except KeyError:
+                LOG.warning('Error when trying to use yaml metrology conf.')
+                LOG.warning('Fallback on the deprecated oslo config method.')
+                bw_data.append(self.t_cloudkitty.format_item(
+                    tap,
+                    self.units_mappings['network.bw.' + direction],
+                    tap_bw_mb,
+                ))
 
         ck_res_name = 'network.bw.{}'.format(direction)
         if not bw_data:
@@ -342,11 +443,76 @@ class CeilometerCollector(collector.BaseCollector):
                                                  floating)
             floating = self._cacher.get_resource_detail('network.floating',
                                                         floating_id)
-            floating_data.append(
-                self.t_cloudkitty.format_item(floating, self.units_mappings[
-                    "network.floating"], 1))
+
+            try:
+                metric = METRICS_CONF['metrics_units']['network.floating']
+                floating_data.append(self.t_cloudkitty.format_item(
+                    floating,
+                    list(metric.values())[0]['unit'],
+                    1,
+                ))
+            # NOTE(mc): deprecated except part kept for backward compatibility.
+            except KeyError:
+                LOG.warning('Error when trying to use yaml metrology conf.')
+                LOG.warning('Fallback on the deprecated oslo config method.')
+                floating_data.append(self.t_cloudkitty.format_item(
+                    floating,
+                    self.units_mappings['network.floating'],
+                    1,
+                ))
+
         if not floating_data:
             raise collector.NoDataCollected(self.collector_name,
                                             'network.floating')
         return self.t_cloudkitty.format_service('network.floating',
                                                 floating_data)
+
+    def get_radosgw_usage(self,
+                          start,
+                          end=None,
+                          project_id=None,
+                          q_filter=None):
+        active_rgw_stats = self.resources_stats('radosgw.objects.size', start,
+                                                end, project_id, q_filter)
+        rgw_data = []
+        for rgw_stats in active_rgw_stats:
+            rgw_id = rgw_stats.groupby['resource_id']
+            if not self._cacher.has_resource_detail('radosgw.usage', rgw_id):
+                raw_resource = self._conn.resources.get(rgw_id)
+                rgw = self.t_ceilometer.strip_resource_data('radosgw.usage',
+                                                            raw_resource)
+                self._cacher.add_resource_detail('radosgw.usage', rgw_id, rgw)
+            rgw = self._cacher.get_resource_detail('radosgw.usage', rgw_id)
+
+            # Unit conversion
+            try:
+                conv_data = METRICS_CONF['metrics_units']['radosgw.usage']
+                rgw_size = ck_utils.convert_unit(
+                    decimal.Decimal(rgw_stats.max),
+                    conv_data['radosgw.object.size'].get('factor', 1),
+                    conv_data['radosgw.object.size'].get('offset', 0),
+                )
+
+                rgw_data.append(
+                    self.t_cloudkitty.format_item(
+                        rgw,
+                        conv_data['rados.objects.size']['unit'],
+                        rgw_size,
+                    )
+                )
+            except KeyError:
+                LOG.warning('Error when trying to use yaml metrology conf.')
+                LOG.warning('Fallback on the deprecated hardcoded method.')
+                rgw_size = decimal.Decimal(rgw_stats.max) / units.Gi
+                rgw_data.append(
+                    self.t_cloudkitty.format_item(
+                        rgw,
+                        self.units_mappings['radosgw.usage'],
+                        rgw_size,
+                    )
+                )
+
+        if not rgw_data:
+            raise collector.NoDataCollected(self.collector_name,
+                                            'radosgw.usage')
+        return self.t_cloudkitty.format_service('radosgw.usage', rgw_data)
