@@ -132,12 +132,18 @@ function configure_cloudkitty {
 
     touch $CLOUDKITTY_CONF
 
-    cp $CLOUDKITTY_DIR$CLOUDKITTY_CONF_DIR/policy.json $CLOUDKITTY_CONF_DIR
+    # generate policy sample file
+    oslopolicy-sample-generator --config-file $CLOUDKITTY_DIR/etc/oslo-policy-generator/cloudkitty.conf --output-file $CLOUDKITTY_DIR/etc/cloudkitty/policy.yaml.sample
+    cp $CLOUDKITTY_DIR/etc/cloudkitty/policy.yaml.sample "$CLOUDKITTY_CONF_DIR/policy.yaml"
+    iniset $CLOUDKITTY_CONF oslo_policy policy_file 'policy.yaml'
+
     cp $CLOUDKITTY_DIR$CLOUDKITTY_CONF_DIR/api_paste.ini $CLOUDKITTY_CONF_DIR
+    cp $CLOUDKITTY_DIR$CLOUDKITTY_CONF_DIR/metrics.yml $CLOUDKITTY_CONF_DIR
     iniset_rpc_backend cloudkitty $CLOUDKITTY_CONF DEFAULT
 
     iniset $CLOUDKITTY_CONF DEFAULT notification_topics 'notifications'
     iniset $CLOUDKITTY_CONF DEFAULT debug "$ENABLE_DEBUG_LOG_LEVEL"
+    iniset $CLOUDKITTY_CONF DEFAULT auth_strategy $CLOUDKITTY_AUTH_STRATEGY
 
     # auth
     iniset $CLOUDKITTY_CONF authinfos auth_type v3password
@@ -160,6 +166,7 @@ function configure_cloudkitty {
     iniset $CLOUDKITTY_CONF collect collector $CLOUDKITTY_COLLECTOR
     iniset $CLOUDKITTY_CONF ${CLOUDKITTY_COLLECTOR}_collector auth_section authinfos
     iniset $CLOUDKITTY_CONF collect services $CLOUDKITTY_SERVICES
+    iniset $CLOUDKITTY_CONF collect metrics_conf $CLOUDKITTY_CONF_DIR/$CLOUDKITTY_METRICS_CONF
 
     # output
     iniset $CLOUDKITTY_CONF output backend $CLOUDKITTY_OUTPUT_BACKEND
@@ -238,6 +245,8 @@ function start_cloudkitty {
         run_process ck-api "$CLOUDKITTY_BIN_DIR/cloudkitty-api --config-file=$CLOUDKITTY_CONF"
     elif is_service_enabled ck-api; then
         enable_apache_site cloudkitty
+        echo_summary "Waiting 15s for cloudkitty-processor to authenticate against keystone before apache is restarted."
+        sleep 15s
         restart_apache_server
         tail_log cloudkitty /var/log/$APACHE_NAME/cloudkitty.log
         tail_log cloudkitty-api /var/log/$APACHE_NAME/cloudkitty_access.log
